/* GraphDistanceComplexity.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2007 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.chemical.indices;

import org.grinvin.GraphBundleView;
import org.grinvin.invariants.RealValue;
import org.grinvin.invariants.computers.AbstractInvariantComputer;

/**
 * computes the graph distance complexity.
 */
public class GraphDistanceComplexity extends AbstractInvariantComputer {
    
    public RealValue compute(GraphBundleView bundle) {
        int[][] distanceMatrix = bundle.distanceMatrix();
        int n = distanceMatrix.length;
        if (n < 1)
            return RealValue.undefinedValue(this);
        int[] vertexDistanceDegree = new int[n];
        int wienerIndex = 0;
        double[] vertexDistanceComplexity = new double[n];
        for(int i=0; i < n; i++) {
            vertexDistanceDegree[i] = distanceMatrix[i][0];
            for(int j=1; j < n; j++) {
                if (distanceMatrix[i][j] == 0 && i!=j)
                    return RealValue.undefinedValue(this); // return undefined for disconnected graphs
                vertexDistanceDegree[i] += distanceMatrix[i][j];
            }
            wienerIndex += vertexDistanceDegree[i];
            for(int j=0; j < n; j++)
                if(distanceMatrix[i][j]!=0)
                    vertexDistanceComplexity[i] -= ((double)distanceMatrix[i][j])/vertexDistanceDegree[i]*Math.log(((double)distanceMatrix[i][j])/vertexDistanceDegree[i])/Math.log(2);
        }
        double index = 0.0;
        for(int i=0; i < n; i++)
            index += vertexDistanceComplexity[i]*vertexDistanceDegree[i];
        index = index/(2*wienerIndex);
        return new RealValue(index, this);
    }
    
    public String getInvariantId() {
        return "org.grinvin.invariants.GraphDistanceComplexity";
    }
}
