/* MeanDistanceDegreeEquality.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2007 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.chemical.indices;

import org.grinvin.GraphBundleView;
import org.grinvin.invariants.RealValue;
import org.grinvin.invariants.computers.AbstractInvariantComputer;

/**
 * computes the mean information content on the distance degree equality of the graph.
 */
public class MeanDistanceDegreeEquality extends AbstractInvariantComputer {
    
    public RealValue compute(GraphBundleView bundle) {
        int[][] distanceMatrix = bundle.distanceMatrix();
        int n = distanceMatrix.length;
        if (n <= 1)
            return RealValue.undefinedValue(this);
        int max = 0;
        int[] vertexDistanceDegree = new int[n];
        for (int i=0; i < n; i++) {
            for (int j=0; j < n; j++) {
                vertexDistanceDegree[i] += distanceMatrix[i][j];
                if (distanceMatrix[i][j] == 0 && i!=j)
                    return RealValue.undefinedValue(this); // return undefined for disconnected graphs
            }
            if (vertexDistanceDegree[i] > max)
                max = vertexDistanceDegree[i];
        }
        double[] vertexDistanceDegreeCount = new double[max+1];
        for(int i=0; i < n; i++)
            vertexDistanceDegreeCount[vertexDistanceDegree[i]]++;
        double index = 0.0;
        double v = n;
        for(int i=0; i<=max; i++)
            if(vertexDistanceDegreeCount[i]!=0)
                index -= vertexDistanceDegreeCount[i]/v*Math.log(vertexDistanceDegreeCount[i]/v)/Math.log(2);
        return new RealValue(index, this);
    }
    
    public String getInvariantId() {
        return "org.grinvin.invariants.MeanDistanceDegreeEquality";
    }
}
