/* TotalDistanceEquality.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2007 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.chemical.indices;

import org.grinvin.GraphBundleView;
import org.grinvin.invariants.RealValue;
import org.grinvin.invariants.computers.AbstractInvariantComputer;

/**
 * computes the total information content on the distance equality of the graph.
 */
public class TotalDistanceEquality extends AbstractInvariantComputer {
    
    public RealValue compute(GraphBundleView bundle) {
        int[][] distanceMatrix = bundle.distanceMatrix();
        int n = distanceMatrix.length;
        if (n <= 1)
            return RealValue.undefinedValue(this);
        int max = distanceMatrix [1][0];
        double[] graphDistanceCount = new double[bundle.getGraph().getNumberOfEdges()+1];
        for (int i=2; i < n; i++) {
            for (int j=0; j < i; j++) {
                graphDistanceCount[distanceMatrix[i][j]]++;
                if (distanceMatrix[i][j] > max)
                    max = distanceMatrix[i][j];
            }
        }
        double v = n;
        double index = v*(v-1)/2*Math.log(v*(v-1)/2)/Math.log(2);
        for(int i=1; i<=max; i++)
            index -= graphDistanceCount[i]*Math.log(graphDistanceCount[i])/Math.log(2);
        return new RealValue(index, this);
    }
    
    public String getInvariantId() {
        return "org.grinvin.invariants.TotalDistanceEquality";
    }
}
